#!/usr/bin/perl

package ConcatMulti;

use Bio::SeqIO;
use Bio::Seq;

#--------------------------------------------------------------------------
# author mt
# concates seqences from a multiple fasta file. position
# indices: 1-length 
#
#---------------------
#OUTPUT:
# index (file index in directory of fasta)
# nr    id    offset_length 
#
# seq file (file seq.concat in directory of fasta) 
# >origin filename
# ATTGAGTA... (without newlines)
#
#
# NOTE: if you change output files, be aware to change resp. 
#      paths in combinate.
#

#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# @param Seqence file in Multiple FASTA
# @return name of concat-file
#-------------------
sub concat{
	my ($self, $seq_file) = @_;
	my ($seq, $input, $output_seq, $count, $offset, $dsp_id);
	$input  = Bio::SeqIO->new('-file'=> $seq_file, '-format'=>"FASTa");
	$seq_file=~ /\/.+\//g;
	$dir=$&;
	if ($'=~ /.+\./){           #fasta with suffix
	    $dsp_id= $&;
	    chop $dsp_id;
	}else{$dsp_id= $'}
	$output_seq=$dir."seq.concat";
	$count = 1;
	$offset = 1;
    	open out_ind, ">".$dir."index" || die "cannot open index file\n";
	open out_seq, ">".$output_seq || die "cannot open concat file\n";
	print out_seq ">".$dsp_id."\n";
	while ($seq = $input->next_seq()){
	    my $id=$seq->display_id();
	    $id=~s/\W/_/g;
	    $id="no_name" unless(defined $id);
	    print out_ind $count."\t".$id."\t"
		.$offset."_".($offset+$seq->length())."\n";
	    print out_seq uc($seq->seq());	    
	    $count++;
	    $offset+=$seq->length()+1; #start of next seq
	}
	print out_seq "\n";
	close out_ind;
	close out_seq;
	return $output_seq;
}
return 1;
