#!/usr/bin/perl

package CriticaNudge;
#=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*
# author mt
# CriticaNudge - Starter class for critica094 or critica105b - defined by 
# property 'critica_home' (parse routine in CriticaParser.pm has to be
# exchanged for critica094 ->commented out)
# offers routines to start the subprograms manually or to start them 
# automated as batch process.
#=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*

@ISA=(Nudge);
$|=1;

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# scriptname is generated from currend time as long to make the name 
# unique
# @param FASTA
# @param OUTPUR_DIR
# @param PREFIX
#---------------------
sub new{
    my $self = shift;
    $self = $self->SUPER::new(shift, shift, shift);
    $self->{RESULT_OUTPUT}=$self->{OUTPUT_DIR}."/".$self->{PREFIX};
    $time = time; 
    $self->{CALL_SCRIPT}="sh_c".$time;
    $self->{BLAST_OUT}= $self->{OUTPUT_DIR}."/".$self->{PREFIX}.".blast";
    $self->{BLAST_PAIRS}=$self->{OUTPUT_DIR}."/".$self->{PREFIX}
                         .".blast.pairs";
    $self->{TRIPLETS}=$self->{OUTPUT_DIR}."/".$self->{PREFIX}.".triplets";
    return $self;
}


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# script for batch processes of all sub 'programs' is written and started
# with qsub.
#-----------------------
sub start_qsub_all{
    my $self = shift;
    $self->print_bash();
    $self->script_blastcontigs();
    $self->script_makeblastpairs();
    $self->script_scanblastpairs();
    $self->script_iteratecritica();
    $self->start_script();
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# writes script for blast-contigs as batch process.
# NOTE for manual calls: print_bash() should be called first.  
#-----------------------
sub script_blastcontigs{
    my $self = shift;
    open(call, ">> ".$self->{OUTPUT_DIR}."/".$self->{CALL_SCRIPT}.".sh") 
         || return 0;
    print call $self->{PARAMETER}{critica_home}."/scripts/blast-contigs ".
	$self->{FASTA}.$self->{PARAMETER}{blastcontigs_db}
        ." > ".$self->{BLAST_OUT}."\n";
    close(call);
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# writes script for make-blastpairs as batch process.
# NOTE for manual calls: print_bash() should be called first.  
#-----------------------
sub script_makeblastpairs{
    my $self = shift;
    open(call, ">> ".$self->{OUTPUT_DIR}."/".$self->{CALL_SCRIPT}.".sh") 
        || return 0;           
    print call $self->{PARAMETER}{critica_home}."/scripts/make-blastpairs "
	 .$self->{BLAST_OUT}.$self->{PARAMETER}{makeblastpairs_exclude}
          ." > "
	    .$self->{BLAST_PAIRS}."\n";
    close(call);
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# writes script for scanblastpairs as batch process.
# NOTE for manual calls: print_bash() should be called first.  
#-----------------------
sub script_scanblastpairs{
    my $self = shift;
    open(call, ">> ".$self->{OUTPUT_DIR}."/".$self->{CALL_SCRIPT}.".sh") 
         || return 0;           
    print call $self->{PARAMETER}{critica_home}."/scanblastpairs "
	       .$self->{FASTA}." ".$self->{BLAST_PAIRS}." "
               .$self->{TRIPLETS}."\n";
    close(call);
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# writes script for iterate-critica as batch process.
# NOTE for manual calls: print_bash() should be called first.  
#-----------------------
sub script_iteratecritica{
    my $self = shift;
    open(call, ">> ".$self->{OUTPUT_DIR}."/".$self->{CALL_SCRIPT}.".sh") 
        || return 0;           
    print call $self->{PARAMETER}{critica_home}."/scripts/iterate-critica"
	.$self->{PARAMETER}{iterate_iterations}
    .$self->{PARAMETER}{iterate_scoringmatrix}
    .$self->{PARAMETER}{iterate_initialdicod}
    .$self->{PARAMETER}{iterate_initialinit}
    .$self->{PARAMETER}{iterate_initialsd}
    .$self->{PARAMETER}{iterate_initialprom}
    .$self->{PARAMETER}{iterate_nosdscores}
    .$self->{PARAMETER}{iterate_promfind}
    .$self->{PARAMETER}{iterate_threshold}
    .$self->{PARAMETER}{iterate_alpha}
    .$self->{PARAMETER}{iterate_fractioncoding}
    .$self->{PARAMETER}{iterate_addlongorfs}
    .$self->{PARAMETER}{iterate_geneticcode}
    .$self->{PARAMETER}{iterate_strictthreshold}
    .$self->{PARAMETER}{iterate_frameshiftthreshold}
    .$self->{PARAMETER}{iterate_quickstats}." "
	.$self->{RESULT_OUTPUT}." ".$self->{FASTA}." "
	    .$self->{TRIPLETS}."\n";
    close(call);
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#@return output file of last iteration. 
#-----------------------
sub get_resultoutput{
    my $self = shift;
    my $i=3;
    if ($self->{PARAMETER}{iterate_iterations}){
	$i = $self->{PARAMETER}{iterate_iterations};}
    return $self->{RESULT_OUTPUT}.$i.".cds";
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# starts blast-contigs as non batch process.
#-----------------------
sub start_blastcontigs{
    my $self = shift;    
    system $self->{PARAMETER}{critica_home}."/scripts/blast-contigs "
	.$self->{FASTA}.$self->{PARAMETER}{blastcontigs_db}
    ." > ".$self->{BLAST_OUT};
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# starts make-blastpairs as non batch process.
#-----------------------
sub start_makeblastpairs{
    my $self = shift;
    system $self->{PARAMETER}{critica_home}."/scripts/make-blastpairs "
	   .$self->{BLAST_OUT}.$self->{PARAMETER}{makeblastpairs_exclude}
           ." > "
	    .$self->{BLAST_PAIRS};
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# starts scanblastpairs as non batch process.
#-----------------------
sub start_scanblastpairs{
    my $self = shift;
    system $self->{PARAMETER}{critica_home}."/scanblastpairs "
	.$self->{FASTA}." ".$self->{BLAST_PAIRS}." ".$self->{TRIPLETS};
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# starts iterate-critica as non batch process.
#-----------------------
sub start_iteratecritica{
    my $self = shift;
    system $self->{PARAMETER}{critica_home}."/scripts/iterate-critica"
	.$self->{PARAMETER}{iterate_iterations}
    .$self->{PARAMETER}{iterate_scoringmatrix}
    .$self->{PARAMETER}{iterate_initialdicod}
    .$self->{PARAMETER}{iterate_initialinit}
    .$self->{PARAMETER}{iterate_initialsd}
    .$self->{PARAMETER}{iterate_initialprom}
    .$self->{PARAMETER}{iterate_nosdscores}
    .$self->{PARAMETER}{iterate_promfind}
    .$self->{PARAMETER}{iterate_threshold}
    .$self->{PARAMETER}{iterate_alpha}
    .$self->{PARAMETER}{iterate_fractioncoding}
    .$self->{PARAMETER}{iterate_addlongorfs}
    .$self->{PARAMETER}{iterate_geneticcode}
    .$self->{PARAMETER}{iterate_strictthreshold}
    .$self->{PARAMETER}{iterate_frameshiftthreshold}
    .$self->{PARAMETER}{iterate_quickstats}." "
	.$self->{RESULT_OUTPUT}." ".$self->{FASTA}." "
	    .$self->{TRIPLETS};
}


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# evaluation of given properties.
# @see critica documentation and ini file.
# @param properties as hash
#----------------------
sub set_parameter{
    my $self = shift;
    $self->{PARAMETER} = shift;
    if (!$self->{PARAMETER}{critica_home}){
	return 0;}

    if ($self->{PARAMETER}{blastcontigs_db}){            #blastcontigs_db 
	$self->{PARAMETER}{blastcontigs_db} = " -db="
                  .$self->{PARAMETER}{blastcontigs_db};}
    if ($self->{PARAMETER}{makeblastpairs_exclude}){     #makeblastpairs_exclude
	$self->{PARAMETER}{makeblastpairs_exclude} = " -exclude=".
	    $self->{PARAMETER}{makeblastpairs_exclude};}

    if ($self->{PARAMETER}{iterate_iterations}){         #iterate_iterations
	$self->{PARAMETER}{iterate_iterations} = " -iterations=".
	    $self->{PARAMETER}{iterate_iterations};}

    if ($self->{PARAMETER}{iterate_scoringmatrix}){      #iterate_scoringmatrix
	$self->{PARAMETER}{iterate_scoringmatrix} = " -scoring-matrix=".
	    $self->{PARAMETER}{iterate_scoringmatrix};}
     
    if ($self->{PARAMETER}{iterate_initialdicod}){      #iterate_initialdicod
	$self->{PARAMETER}{iterate_initialdicod} = " -initial-dicod=".
	    $self->{PARAMETER}{iterate_initialdicod};}

    if ($self->{PARAMETER}{iterate_initialinit}){        #iterate_initialinit
	$self->{PARAMETER}{iterate_initialinit} = " -initial-init=".
	    $self->{PARAMETER}{iterate_initialinit};}

    if ($self->{PARAMETER}{iterate_initialsd}){          #iterate_initialsd
	$self->{PARAMETER}{iterate_initialsd} = " -initial-sd=".
	    $self->{PARAMETER}{iterate_initialsd};}	

    if ($self->{PARAMETER}{iterate_initialprom}){        #iterate_initialprom
	$self->{PARAMETER}{iterate_initialprom} = " -initial-proms=".
	    $self->{PARAMETER}{iterate_initialprom};}

    if ($self->{PARAMETER}{iterate_nosdscores} eq "true"){       #iterate_nosdscores
	$self->{PARAMETER}{iterate_nosdscores} = " -no-sdscores";}	
    else { undef $self->{PARAMETER}{iterate_nosdscores};}

    if ($self->{PARAMETER}{iterate_promfind} eq "true"){         #iterate_promfind
	$self->{PARAMETER}{iterate_promfind} = " -prom-find";}
    else { undef $self->{PARAMETER}{iterate_promfind};}    

    if ($self->{PARAMETER}{iterate_threshold}){          #iterate_threshold
	$self->{PARAMETER}{iterate_threshold} = " -threshold=".
	    $self->{PARAMETER}{iterate_threshold};}

    if ( $self->{PARAMETER}{iterate_alpha}){              #iterate_alpha
	$self->{PARAMETER}{iterate_alpha} = " -alpha=".
	    $self->{PARAMETER}{iterate_alpha};}
    
    if ( $self->{PARAMETER}{iterate_fractioncoding}){     #iterate_fractioncoding
	$self->{PARAMETER}{iterate_fractioncoding} = " -fraction-coding=".
	    $self->{PARAMETER}{iterate_fractioncoding};}
    
    if ( $self->{PARAMETER}{iterate_addlongorfs}){        #iterate_addlongorfs
	$self->{PARAMETER}{iterate_addlongorfs} = " -add-longorfs=".
	    $self->{PARAMETER}{iterate_addlongorfs};}

    if ( $self->{PARAMETER}{iterate_geneticcode}){        #iterate_geneticcode
	$self->{PARAMETER}{iterate_geneticcode} = " -genetic-code=".
	    $self->{PARAMETER}{iterate_geneticcode};}

    if ($self->{PARAMETER}{iterate_strictthreshold} eq "true"){  #iterate_strictthreshold
	$self->{PARAMETER}{iterate_strictthreshold} = " -strict-threshold";}
    else {undef $self->{PARAMETER}{iterate_strictthreshold};}

    if ($self->{PARAMETER}{iterate_frameshiftthreshold}){#iterate_frameshiftthreshold
	$self->{PARAMETER}{iterate_frameshiftthreshold} = " -frameshift-threshold=".
	    $self->{PARAMETER}{iterate_frameshiftthreshold};}

    if ($self->{PARAMETER}{iterate_quickstats} eq "true"){       #iterate_quickstats
	$self->{PARAMETER}{iterate_quickstats} = " -quick-stats";}
    else {undef $self->{PARAMETER}{iterate_quickstats};}
    return 1;
}
return 1;
