#!/usr/bin/perl

package CriticaParser;

#=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*
# author mt
# Parser for critica output files (tested for version 094 - 105b)
# NOTE:
# critica gives end position of putative genes with last index of stopcodon
#       (inclusive terminal codon), start is given as first base of 
#        startcodon)
# start is given as first base of the startcodon  
#=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*
use YACOP::gene_pred::DataHandle;

my $TYPE = "CriticaParser";
my $NAME = "Critica";
my $REF_TYPE = "genebank";
my $DIFF = 2; #difference between YACOP critica notation of stopcodon

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# CRITICA105b
# Whole data from given file is read and stored as Element-objects to a 
# new created DataHandle. 
# end positions are changed to yacop notation by $DIFF nucleotides 
#     (first index of terminal codon)  
# score is p-value: amount of statistical support (lower are better) 
# comment of Element-Objects is here shine dalgano if available
# Note: orfs from critica are not in descending order!
#@param filename of critica-output
#@return new DataHandle
#---------------------------------
sub parse{
    my ($self, $file) = @_;
    my $data_handle = DataHandle->new($TYPE);
    my (@array, $i, $diff);
    $i = 1;
    open (critica_input, $file) 
	|| die "can not open critica input file ".$file."\n";
    while (<critica_input>){
	@array = split; 
	my $com = $self->complement($array[1], $array[2]);
	if($com=~/R/i){$diff=$DIFF;}
	else{$diff=-$DIFF;}
	$data_handle->add_element($i, $array[1], ($array[2]+$diff), $com, 
				  $array[3], $array[11]);
	$i++;
    }
    close(critica_input);
    return $data_handle;
}


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Calulates the complement of given start - end position. 
# @param $start
# @param $end
# @return complement 
#---------------------------
sub complement{
    my ($self, $start, $end) = @_;
    return 'F' if($start<$end);
    return 'R';
}

sub name{
    return $NAME;
}

sub type{
    return $TYPE;
}
return 1;


