#!/usr/bin/perl

package DataHandle;
#=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*
# author mt
# data structure to hold Element-objects (ORFs), with posibillity to sort 
# them. 
#=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*
use YACOP::gene_pred::Element;
use YACOP::gene_pred::ResultSet;
use YACOP::gene_pred::StopCodon;

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# CONSTRUCTOR.
# @param id (in this case parser type)
#-----------------------
sub new{
    my $self = { };
    bless $self, shift;
    $self->id(shift);    
    $self->{POS_INDEX} = 1;
    $self->{COUNTER} = 1;
    $self->{POINTER} = 1;
    return $self;
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# NOTE: for use with meta_pred.pl: DataHandle id is the same as parser type
#@param id 
#@return id (mostly an integer) 
#-----------------------
sub id{
    my ($self, $tmp) = @_;
    $self->{ID} = $tmp if($tmp);
    return $self->{ID};    
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# NOTE: counter is initialized with 1 not 0
#@return current number of elements as integer 
#-----------------------
sub counter{
    my $self=shift;
    return $self->{COUNTER}-1;
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Adding new Element to current dataHandle.
#@param id 
#@param start (should be integer)
#@param end (should be integer) 
#@param score 
#@param comment 
#-----------------------
sub add_element{
     my $self = shift;
     $self->{ELEMENT}{$self->{POS_INDEX}++} = 
	 Element->new($_[0], $_[1], $_[2], $_[3], $_[4], $_[5]);
     $self->{COUNTER}++;
}


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#removing element from datahandle and decreases the counter.
#---------------------
sub remove_element{
    my ($self, $id)=@_;
    if($self->{ELEMENT}{$id}){
	undef $self->{ELEMENT}{$id};
	$self->{COUNTER}--;
    }
    return $self->{COUNTER};
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#
#---------------------
sub add_stats{
    my ($self, $tmp) = @_;
    $self->{$tmp}++ if($tmp);
    return $self->{$tmp};
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#
#---------------------
sub get_stats{
    my ($self, $tmp) = @_;
    return $self->{$tmp};
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#getting element by id.  
#---------------------
sub get_element{
    my ($self, $id) = @_;
    return $self->{ELEMENT}{$id};
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#getting all elements as hash.
#@return hash holding the elements  
#---------------------
sub get_elements{
    my $self = shift;
    return %{$self->{ELEMENT}};
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Sorting (not very efficiently) ids of Element-objects ascending 
# (from end positions) 
# NOTE: DataHandle ids (which are the keys, are changed to acending 
# numbers. internal Element ids are untouched. 
#-----------------------
sub sort_elements{
    my $self = shift;
    my @array;
    my @elements = values %{$self->{ELEMENT}};
    undef %{$self->{ELEMENT}};
    my $i = 0;
    foreach (@elements){$array[$i++] = $_->end;}
    @array = sort {$a <=> $b} @array; # sorting the array ascending
    $i = 1;
    foreach (@array){
	my $tmp = $_ ;
	foreach (@elements){
	    if ($_ && $_->end == $tmp){
		$self->{ELEMENT}{$i++}= $_;
		undef $_;
	    }
	}
    }
    return;
}

return 1;


