#!/usr/bin/perl

package GlimmerParser;

#=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*
# author mt
# Parser for glimmer output files (tested for version 2.02 and 2.10)

# NOTE: glimmer gives end position of putative genes as last position 
#       before stop (->exclusive terminal codon) 
#  start codons: atg, ctg, gtg and ttg
# start is given as first base of the startcodon  
#=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*
use YACOP::gene_pred::DataHandle;

my $TYPE = "GlimmerParser"; 
my $NAME = "Glimmer";
my $DIFF=1;

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Whole data from given file is read and stored as Element-objects to a new
# created DataHandle. Start is the first base of start codon, end position 
# is before terminator.
# start: is the first base of the startcodon 
# end: is 1 base before stop, this is changed to the yacop end-position by 
#     $DIFF to first position of stopcodon.
# score: probability thate the markov model is in the correct frame 
# comment is here direction of the gene (F/R). 
#@param filename of glimmer-output
#@return current DataHandle 
#------------------------
sub parse{
    my ($self, $file) = @_;
    my $data_handle = DataHandle->new($TYPE);
    my  (@array, $c, $comm, $length);
    open(glim_input, $file) 
       || die "cannot open glimmer input file ".$file."\n";
    while (<glim_input>){last if(/putative/i);}
    while (<glim_input>){            #from putative genes
	$comm = '-';
	@array = split;
	$e = $data_handle->get_element($array[0]);
	if (@array > 6){    #comment...
	    for(my $i=6; $i<@array; $i++){$comm.=$array[$i];}}
	if ($e){
	    $e->start($array[1]);
	    $e->end($array[2]);
	    $e->comment($comm);}
	else {                             #shouln't be more than 1 or 2 
	    $array[3]=~ /\d/;              #possible if genome start 
	    $c = $&;                       #is in the pred. gene
	    if ($array[3] =~ /(-)/){
		$c = "R";
		$diff=-$DIFF;
	    }
	    else{
		$c = "F";
		$diff=$DIFF;
	    }
	    $data_handle->add_element($array[0], $array[1],
				      ($array[2]+$diff), $c, '-', $comm);		
	}
    }
    close(glim_input);
    return $data_handle;
}

sub name{
    return $NAME;
}

sub type{
    return $TYPE;
}
return 1;

