#!/usr/bin/perl

package ZCurveParser;

#=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*
# author mt
# Parser for zcurve output files
# NOTE:  zcurve gives end position of putative genes as last position 
#         before stop
#       (->exclusive terminal codon) 
#  start codons: atg, ctg, gtg and ttg
# start is given as first base of the startcodon  
#=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*
use YACOP::gene_pred::DataHandle;

my $TYPE = "ZCurveParser"; 
my $NAME = "ZCURVE";

my $DIFF=2;     #difference ZCurve output and yacop notation

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Whole data from given file is read and stored as Element-objects to a new
# created DataHandle. Start is the first base of start codon, end position 
# is before terminator.
# start ist the first base of the startcodon (remains unchanged)
# end 1 base before stop (changed by $DIFF nucleotides to first nucleotid
# of stopcodon.
# score: probability thate the markov model is in the correct frame 
# comment is here direction of the gene (F/R). 
#@param filename of zcurve-output
#@return current DataHandle 
#------------------------
sub parse{
    my ($self, $file) = @_;
    my $data_handle = DataHandle->new($TYPE);
    my  (@array, $c, $comm, $length, $start, $stop);
    open(zcurve_input, $file) 
	|| die "cannot open zcurve input file ".$file."\n";
    while (<zcurve_input>){last if (/No/i);}
    while (<zcurve_input>){            #from putative genes
	if (/\S/){
	    @array = split;
	    if ($array[3] == 1){
		$c='F';
		$start=$array[1];
		$stop=$array[2]-$DIFF; 
	    }
	    else{
		$c='R';
		$start=$array[2];
		$stop=$array[1]+$DIFF;
	    }
	    $data_handle->add_element($array[0], $start, $stop, $c, 
				  $array[5], '-');		
	}
    }
    close(zcurve_input);
    return $data_handle;
}

sub name{
    return $NAME;
}

sub type{
    return $TYPE;
}
return 1;







